@@ -1,14 +1,13 @@
 import os
 import subprocess
 import tempfile
-import time
 
 import gi
 
 from pmos_tweaks.settingstree import SettingsTree
 
 gi.require_version('Gtk', '3.0')
-from gi.repository import Gtk, GLib, GObject, Gio, Gdk, GLib, Pango
+from gi.repository import Gtk, Gdk, Pango
 
 gi.require_version('Handy', '1')
 from gi.repository import Handy
@@ -125,7 +124,8 @@ class TweaksWindow(Handy.ApplicationWindow):
         self.action_revealer = Gtk.Revealer()
         box.pack_start(self.action_revealer, False, True, 0)
         self.action_revealer.add(self.actionbar)
-        label = Gtk.Label(label="You have changed settings that need root permissions to save.", xalign=0.0)
+        label = Gtk.Label(label="You have changed settings that need root"
+                          " permissions to save.", xalign=0.0)
         label.set_line_wrap(True)
         self.actionbar.pack_start(label)
         self.action_button = Gtk.Button.new_with_label("Apply")
@@ -183,6 +183,25 @@ class TweaksWindow(Handy.ApplicationWindow):
 
                 for name in self.settings.settings[page]['sections'][section]['settings']:
                     setting = self.settings.settings[page]['sections'][section]['settings'][name]
+
+                    try:
+                        # This throws if the backend encounters an error
+                        value = setting.get_value()
+                    except Exception:
+                        error_label = Gtk.Label(label="{name}: exception occurred loading this"
+                                                " setting", xalign = 0.0)
+                        import traceback
+                        tb = traceback.format_exc()
+                        copy_details_button = Gtk.Button(label="Copy exception trace")
+                        copy_details_button.connect('clicked', lambda self: \
+                                Gtk.Clipboard.get_default(self.get_display()).set_text(tb, -1))
+                        info_bar = Gtk.InfoBar()
+                        info_bar.set_message_type(Gtk.MessageType.ERROR)
+                        info_bar.get_content_area().add(error_label)
+                        info_bar.add_action_widget(copy_details_button, 0)
+                        frame.add(info_bar)
+                        continue
+
                     sbox = Gtk.Box()
                     sbox.set_margin_top(8)
                     sbox.set_margin_bottom(8)
@@ -205,7 +224,7 @@ class TweaksWindow(Handy.ApplicationWindow):
 
                     if setting.type == 'boolean':
                         widget = Gtk.Switch()
-                        widget.set_active(setting.get_value())
+                        widget.set_active(value)
 
                         # Make sure I leak some memory
                         setting.widget = widget
@@ -216,7 +235,7 @@ class TweaksWindow(Handy.ApplicationWindow):
                         setting.connect(self.on_setting_change)
                         wbox.pack_start(widget, False, False, 0)
                     elif setting.type == 'info':
-                        widget = Gtk.Label(label=setting.get_value())
+                        widget = Gtk.Label(label=value)
                         widget.set_xalign(0.0)
                         widget.get_style_context().add_class('dim-label')
                         widget.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
@@ -227,11 +246,10 @@ class TweaksWindow(Handy.ApplicationWindow):
                         widget.setting = setting
 
                         widget.set_entry_text_column(0)
-                        val = setting.get_value()
                         i = 0
                         for key in setting.map:
                             widget.append_text(key)
-                            if key == val:
+                            if key == value:
                                 widget.set_active(i)
                             i += 1
                         widget.connect('changed', self.on_widget_changed)
@@ -241,7 +259,7 @@ class TweaksWindow(Handy.ApplicationWindow):
                         widget = Gtk.FontButton()
                         setting.widget = widget
                         widget.setting = setting
-                        widget.set_font(setting.get_value())
+                        widget.set_font(value)
                         widget.connect('font-set', self.on_widget_changed)
                         setting.connect(self.on_setting_change)
                         wbox.pack_start(widget, False, False, 0)
@@ -249,14 +267,12 @@ class TweaksWindow(Handy.ApplicationWindow):
                         widget = Gtk.FileChooserButton()
                         setting.widget = widget
                         widget.setting = setting
-                        temp = setting.get_value()
-                        if temp is None:
-                            temp = '~'
-                        widget.set_filename(temp)
+                        filename_to_display = value or '~'
+                        widget.set_filename(filename_to_display)
                         widget.connect('file-set', self.on_widget_changed)
                         setting.connect(self.on_setting_change)
                         enable = Gtk.Switch()
-                        if temp != '~':
+                        if filename_to_display != '~':
                             enable.set_active(True)
                         enable.setting = setting
                         enable.set_margin_bottom(5)
@@ -271,7 +287,6 @@ class TweaksWindow(Handy.ApplicationWindow):
                         widget = Gtk.ColorButton()
                         setting.widget = widget
                         widget.setting = setting
-                        value = setting.get_value()
                         if value.startswith('#'):
                             color = Gdk.color_parse(value)
                             widget.set_color(color)
@@ -279,13 +294,14 @@ class TweaksWindow(Handy.ApplicationWindow):
                         setting.connect(self.on_setting_change)
                         wbox.pack_start(widget, False, False, 0)
                     elif setting.type == 'number':
-                        value = setting.get_value()
+                        value_to_display = value
                         if 'percentage' in setting.definition and setting.definition['percentage']:
                             w_min = 0
                             w_max = 100
                             w_step = 1
                             val_range = setting.definition['max'] - setting.definition['min']
-                            value = int((value - setting.definition['min']) / val_range * 100)
+                            value_to_display = int((value - setting.definition['min']) \
+                                                   / val_range * 100)
 
                         else:
                             w_min = setting.definition['min']
@@ -294,8 +310,8 @@ class TweaksWindow(Handy.ApplicationWindow):
                         widget = Gtk.SpinButton.new_with_range(w_min, w_max, w_step)
                         setting.widget = widget
                         widget.setting = setting
-                        if value is not None:
-                            widget.set_value(float(value))
+                        if value_to_display is not None:
+                            widget.set_value(float(value_to_display))
                         widget.connect('value-changed', self.on_widget_changed)
                         setting.connect(self.on_setting_change)
                         wbox.pack_start(widget, False, False, 0)
